import time
import datetime as dt


def wait(target_time):
    """Wait until a specified time.

    Args:
        target_time (str): Time in ISO format to wait until,
            i.e. "2015-10-21T07:28:00"

    """
    t0 = dt.datetime.now()
    t1 = dt.datetime.fromisoformat(target_time)

    assert t1 > t0, f"time {t1} is in the past"

    diff = t1-t0
    print(f"Waiting for {diff.total_seconds()} seconds")
    time.sleep(diff.total_seconds())



from . import acu, seq, smurf

from .commands import wait
from .util import create_clients

CLIENTS = None


def initialize():
    global CLIENTS
    CLIENTS = create_clients()


__all__ = ["acu", "seq", "smurf", "wait", "initialize"]

from . import _version
__version__ = _version.get_versions()['version']



import sorunlib as run


def move_to(az, el, wait=None):
    """Move telescope to specified coordinates.

    Args:
        az (float): destination angle for the azimuthal axis
        el (float): destination angle for the elevation axis
        wait (float): amount of time to wait for motion to end

    """
    run.CLIENTS['acu'].go_to(az, el, wait)



import os

from ocs import site_config
from ocs.ocs_client import OCSClient


def _load_site_config(filename=None):
    """Load a site config file, searching for default.yaml in OCS_CONFIG DIR by
    default.

    Args:
        filename (str): Path to OCS site config file

    Returns:
        ocs.site_config.SiteConfig object

    """
    if filename is None:
        assert (os.getenv('OCS_CONFIG_DIR') is not None)
        site = 'default'
        filename = os.path.join(os.getenv('OCS_CONFIG_DIR'),
                                site + '.yaml')
    cfg = site_config.SiteConfig.from_yaml(filename)

    return cfg


def _find_instances(agent_class, host=None, config=None):
    """Find all instances of an Agent Class in a config.

    Args:
        agent_class (str): Agent Class name to search for, must match Agent
            Class defined by an OCS Agent (and thus also defined in the SCF.)
        host (str): Specific host to search on, if None all hosts will be
            searched.
        config (str): Path to the OCS Site Config File. If None the default
            file in OCS_CONFIG_DIR will be used.

    Returns:
        list: List of instance-id's matching the given agent_class.

    """
    cfg = _load_site_config(config)

    if host is not None:
        hosts = {host: cfg.hosts[host]}
    else:
        hosts = cfg.hosts

    instances = []
    for _host, hostcfg in hosts.items():
        for entry in hostcfg.instances:
            if entry['agent-class'] == agent_class:
                instances.append(entry['instance-id'])

    return instances


def create_clients(config=None):
    """Create all clients needed for commanding a single platform.

    Args:
        config (str): Path to the OCS Site Config File. If None the default
            file in OCS_CONFIG_DIR will be used.

    Returns:
        dict: Dictionary with the ACU and SMuRF clients needed for commanding
        in the format::

            clients = {'acu': acu_client,
                       'smurf': [smurf_client1, smurf_client2, smurf_client3]}

    """
    acu_id = _find_instances('ACUAgent', config)
    smurf_ids = _find_instances('PysmurfController', config)

    acu_client = OCSClient(acu_id[0])
    smurf_clients = [OCSClient(x) for x in smurf_ids]

    clients = {'acu': acu_client,
               'smurf': smurf_clients}
    return clients



import sorunlib as run


def bias_step():
    """Perform a bias step on all SMuRF Controllers"""
    for smurf in run.CLIENTS['smurf']:
        smurf.run.start('bias_step.sh')

    for smurf in run.CLIENTS['smurf']:
        smurf.run.wait()


def iv_curve():
    """Perform a bias step on all SMuRF Controllers"""
    for smurf in run.CLIENTS['smurf']:
        smurf.run.start('iv_curve.sh')

    for smurf in run.CLIENTS['smurf']:
        smurf.run.wait()
